atom
	proc
		Width() . = TILE_WIDTH
		Height() . = TILE_HEIGHT
		Px(P) . = (x-1)*TILE_WIDTH  + (P && P*Width())
		Py(P) . = (y-1)*TILE_HEIGHT + (P && P*Height())
		Cx() . = Px(1/2)
		Cy() . = Py(1/2)

		Position(Xp, Yp) . = vec2(Px(Xp), Py(Yp))
		Center() . = Position(1/2, 1/2)
		DistanceTo(atom/A) . = vec2_magnitude(vec2_to(Center(), A.Center()))
		AngleTo(atom/A) . = vec2_angle_to(Center(), A.Center())

	movable
		var _decimal[]

		Width() . = bound_width
		Height() . = bound_height
		Px(P) . = ..() + bound_x + step_x
		Py(P) . = ..() + bound_y + step_y

		proc
			Translate(X, Y)
				if(isvec2(X))
					Y = X[2]
					X = X[1]
				if(!(X || Y)) return
				if(!_decimal) _decimal = vec2_zero
				var rx, ry
				if(X)
					_decimal[1] += X
					rx = round(_decimal[1], 1)
					_decimal[1] -= rx
				if(Y)
					_decimal[2] += Y
					ry = round(_decimal[2], 1)
					_decimal[2] -= ry
				var s = step_size
				step_size = max(abs(rx), abs(ry)) + 1
				. = (rx || ry) ? Move(loc, dir, step_x + rx, step_y + ry) : TRUE
				step_size = s

			Slide(X, Y)
				if(isvec2(X))
					Y = X[2]
					X = X[1]
				if(!Translate(X, Y))
					if(Translate(X, 0))
						Translate(0, Y)
					else if(Translate(0, Y))
						Translate(X, 0)

			SetLoc(Loc, StepX, StepY)
				loc = Loc
				if(!isnull(StepX)) step_x = StepX
				if(!isnull(StepY)) step_y = StepY

			SetPosition(X, Y, Z)
				if(isvec2(X))
					Z = Y
					Y = X[2]
					X = X[1]
				if(!(isnum(X) && isnum(Y))) throw EXCEPTION("bad number")
				if(isnull(Z)) Z = z

				/*
				// Bug: step_x && step_y are 16-bit integers with absolute limit of 32768, so this won't work at locations too far from (1, 1, z).
				loc = locate(1, 1, Z)
				step_x = X - 1 - bound_x
				step_y = Y - 1 - bound_y
				Move(loc, dir, step_x, step_y) // normalizes step_x && step_y
				*/

				// Bug: placement may fail if new loc is off the map, even if it should still be on the map
				loc = locate(1 + X / tile_width(), 1 + Y / tile_height(), Z)
				step_x = X % tile_width() - bound_x
				step_y = Y % tile_height() - bound_y

			SetCenter(X, Y, Z)
				if(isvec2(X))
					Z = Y
					Y = X[2]
					X = X[1]
				else if(isloc(X))
					var atom/a = X
					X = a.Cx()
					Y = a.Cy()
					Z = a.z
				SetPosition(X - Width()/2, Y - Height()/2, Z)