var gravity = vec2(0, -0.05)

proc
	spawn_planes(N, Z = 1)
		for(var/n in 1 to N)
			new /enemy_plane (locate(rand(1, world.maxx), rand(1, world.maxy), Z))

world
	New()
		..()
		spawn(50) spawn_planes(10)

plane
	parent_type = /entity
	icon = 'plane parts.dmi'
	icon_state = "body"

	bounds = "16,16"
	bound_x = 8
	bound_y = 8

	var
		matrix/shape = matrix()

		velocity[] = vec2_zero
		mass = 1

		thrust = 0.3
		max_speed = 7
		friction = 0.999

		angle = 0
		max_turn_rate = 3
		max_boost_turn_rate = 2

		tmp/next_shot
		shot_cooldown = 1
		shot_speed = 12
		shot_color = rgb(255, 255, 204)
		shot_border
		shot_damage = 1

		armor

		turn_rate
		boosting = FALSE
		shooting = FALSE

		image
			tail
			tail_up
			back_wing
			front_wing

	New()
		..()
		UpdateShape()

	Update()
		turn_rate && Turn(turn_rate)
		boosting && Thrust()
		gravity && Gravity()
		friction && Friction()
		Speed() > max_speed && LimitSpeed()
		Speed() > 1e-2 && Slide(velocity)
		shooting && CanShoot() && Shoot()

	Shot(projectile/P)
		PlaySound('hit.wav', 200)
		TakeDamage(P.damage)

	proc
		CanShoot() return world.time >= next_shot
		Speed() return vec2_magnitude(velocity)

		AddForce(Force) velocity = vec2_add(velocity, vec2_scale(Force, 1/mass))
		LimitSpeed() velocity = vec2_scale_to(velocity, max_speed)
		Gravity() velocity = vec2_add(velocity, gravity)
		Friction() velocity = vec2_scale(velocity, friction)
		Thrust() AddForce(vec2p(thrust, angle))

		Turn(Angle)
			angle += Angle
			UpdateShape()

		TakeDamage(Damage)
			if(armor > 0)
				var c = color
				animate(src, color = "white", time = 1)
				animate(color = c, time = 1)
				armor -= Damage
				if(armor <= 0)
					Die()

		Die()

		Shoot()
			next_shot = world.time + shot_cooldown
			var projectile/p = new (src)
		//	p.velocity = vec2_add(velocity, vec2p(shot_speed, angle + randn(-5, 5)))
			p.velocity = vec2p(shot_speed, angle + randn(-5, 5))
			AddForce(vec2p(0.2, angle + 180))

		UpdateShape()
			transform = turn(shape, angle)

			if(!overlays.Remove(tail)) tail = image('plane parts.dmi', "tail", FLOAT_LAYER)
			if(!overlays.Remove(tail_up)) tail_up = image('plane parts.dmi', "tail up", FLOAT_LAYER)
			if(!underlays.Remove(back_wing)) back_wing = image('plane parts.dmi', "wing", FLOAT_LAYER)
			if(!overlays.Remove(front_wing)) front_wing = image('plane parts.dmi', "wing", FLOAT_LAYER)

			tail.transform = matrix(abs(cos(angle)), 0, 0, 0, 1, -30)
			tail_up.transform = matrix(-sin(angle), 0, 0, 0, 1, -30)

			var matrix/wing_pos = matrix(1, 0, 16, 0, 1, 0)
			var matrix/s = matrix(abs(cos(angle)), 0, 0, 0, 1, 0)
			back_wing.transform = wing_pos * matrix(-1, 0, 0, 0, 1, 0) * s
			front_wing.transform = wing_pos * s

			underlays += back_wing
			overlays.Add(tail, tail_up, front_wing)

player_plane
	parent_type = /plane
	color = "maroon"

	shot_color = rgb(255, 255, 150)
	shot_border = rgb(200, 200, 150)

	var
		tmp
			spawning_enemies
			next_melee_check
			camera_position[2]
		melee_cooldown = 0.5
		melee_damage = 0.5

	Login()
		src << sound('burner.ogg', repeat = TRUE)
		SetCenter(map_width()/2, map_height()/2, 1)
		..()
		SetupKeyboard()

	Update()
		PlayerControl()
		CheckMelee()
		..()
		UpdateCamera()
	//	ShowCPU()

	verb
		spawn_10() spawn_planes(10)

	proc
		ShowCPU() winset(src, ":window", "title=\"[world.name] ([world.cpu])\"")

		SetupKeyboard()
			new /keyboard (client)
			client.keyboard.Axis("turn", "east", "west")
			client.keyboard.Axis("boost", "north")
			client.keyboard.Axis("shoot", "x")

		PlayerControl()

			turn_rate = client.keyboard.Check("turn") * (boosting ? max_boost_turn_rate : max_turn_rate)
			boosting = client.keyboard.Check("boost")
			shooting = client.keyboard.Check("shoot")

		CheckMelee()
			if(world.time < next_melee_check) return
			next_melee_check = world.time + melee_cooldown
			for(var/enemy_plane/e in obounds(16)) e.TakeDamage(melee_damage)

		UpdateCamera()
			client.FixCamera()
			camera_position = vec2_lerp(camera_position, vec2p(min(client.ViewPixelWidth(), client.ViewPixelHeight()) / 3, angle), 0.01)
			client.pixel_x += camera_position[1]
			client.pixel_y += camera_position[2]

var enemies[0]

enemy_plane
	parent_type = /plane
	color = "maroon"

	armor = 2

	max_turn_rate = 1
	max_speed = 7.5

	shot_cooldown = 30
	shot_border = "maroon"

	shape = matrix(4/5, 0, 0, 0, 4/5, 0)

	New()
		..()
		max_turn_rate += randn(-0.2, 0.2)
		max_speed += randn(-0.1, 0.1)
		enemies += src

	Del()
		enemies -= src
		..()

	Update()
		var player_plane/p = locate()
		if(p)
			var angle_diff = angle_difference(AngleTo(p), angle)
			var abs_diff = abs(angle_diff)
			turn_rate = abs_diff <= max_turn_rate ? angle_diff : sign(angle_diff) * max_turn_rate
			boosting = abs_diff <= 90
			shooting = abs_diff <= 20
		..()

	Shoot()
		shot_cooldown = initial(shot_cooldown) + randn(0, 20)
		..()

	Die()
		if(enemies.len <= 10)
			new /enemy_plane (locate(rand(1, world.maxx), rand(1, world.maxy), z))
		del src